/*
 * Carsten Langgaard, carstenl@mips.com
 * Copyright (C) 2000 MIPS Technologies, Inc.  All rights reserved.
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 */
// #include <linux/config.h>
#include <linux/autoconf.h>
#include <linux/undefconf.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/ioport.h>
#include <linux/pci.h>
#include <linux/tty.h>

#include <linux/serial.h>
#include <linux/serial_core.h>
#include <linux/serial_8250.h>


#ifdef CONFIG_MTD
#include <linux/mtd/partitions.h>
#include <linux/mtd/physmap.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#endif

#include <asm/cpu.h>
#include <asm/bootinfo.h>
#include <asm/irq.h>
#include <asm/mips-boards/generic.h>
#include <asm/mips-boards/prom.h>
#include <asm/dma.h>
#include <asm/time.h>
#include <asm/traps.h>

#include "chip_int.h"
#include "hwreg_S5.h"
#ifdef CONFIG_VT
#include <linux/console.h>
#endif

#include "board/Board.h"


const char display_string[] = "        LINUX ON Titania3     ";

extern void mips_reboot_setup(void);
extern void mips_time_init(void);
extern void mips_timer_setup(struct irqaction *irq);
extern unsigned long mips_rtc_get_time(void);

static void __init serial_init(void);

const char *get_system_type(void)
{
	return "MStar Titania3";
}

static int __init Titania3_setup(void)
{
       // int i;

	printk("Begin Titania3_setup\n");

    (*(volatile unsigned char *)(0xBF2025C0))=(unsigned char)(0x0);//for SWUpdateDelayCode for T3 only.

	serial_init();

	mips_reboot_setup();

	return 0;
}

void __init plat_mem_setup(void)
{
    Titania3_setup();

}

static int __init mem_map_miu1(void)
{
    // === from mpool
    unsigned long flags;
    unsigned long entrylo0 = ((0x40000000>> 12) << 6) | (0<<3) | (1<< 2) | (1<< 1) | (1<< 0); // cacheable
    unsigned long entrylo1 = ((0x40000000>> 12) << 6) | (2<<3) | (1<< 2) | (1<< 1) | (1<< 0); // un-cacheable or 0x7
    local_irq_save(flags);
    add_wired_entry(entrylo0, entrylo1, 0xC0000000, PM_256M);
    local_irq_restore(flags);

	return 0;
}
__initcall(mem_map_miu1);



extern void Uart16550IntrruptEnable(void);
extern void Uart16550Put(unsigned char);
static void __init serial_init(void)
{
#ifdef CONFIG_SERIAL_8250
	struct uart_port s,u,f;

	memset(&s, 0, sizeof(s));

    Uart16550IntrruptEnable();
	//while(1) Uart16550Put('*');
	s.type = PORT_16550;
	s.iobase = 0xbf201300;
	// s.irq = E_IRQ_UART;
	s.irq = E_IRQ_UART0;

#ifdef CONFIG_MSTAR_TITANIA3

	// for FPGA
	// s.uartclk   = 12000000;
	//s.uartclk = 14318180;
	s.uartclk = 123000000;
	//s.uartclk = 144000000;
	//s.uartclk = 160000000;
	//s.uartclk = 172800000;
	//s.uartclk = 108000000;

#else

#ifdef CONFIG_MSTAR_TITANIA3_BD_FPGA
	//s.uartclk = FPGA_XTAL_CLOCK;    //12000000;///14318180;   //43200000;
	s.uartclk = 12000000 ;
#else
	s.uartclk = 123000000;
#endif

#endif
	s.iotype = 0;
	s.regshift = 0;
	s.fifosize = 16 ; // use the 8 byte depth FIFO well

	if (early_serial_setup(&s) != 0) {
		printk(KERN_ERR "Serial setup failed!\n");
	}

	// ------->hank.lai 2010.2.3
	memset(&u, 0, sizeof(u));

	u.type = PORT_16550;
	u.iobase = 0xBF220C00;
	u.irq = E_IRQEXPL_UART1;
	u.uartclk = 108000000; //@@@
	u.iotype = 0;
	u.regshift = 0;
	u.fifosize = 16 ; // use the 8 byte depth FIFO well

	u.line = 1;

	if (early_serial_setup(&u) != 0) {
		printk(KERN_ERR "Serial piu_uart1 setup failed!\n");
	}
	// <-------

	// ------->hank.lai 2010.2.11
	memset(&f, 0, sizeof(f));

	f.type = PORT_16550;
	f.iobase = 0xBF220D00;
	f.irq = E_IRQEXPH_UART2MCU;
	f.uartclk = 123000000;
	f.iotype = 0;
	f.regshift = 0;
	f.fifosize = 16 ; // use the 8 byte depth FIFO well

	f.line = 2;

	if (early_serial_setup(&f) != 0) {
		printk(KERN_ERR "Serial piu_fuart setup failed!\n");
	}
	// <-------
#endif
}

extern unsigned long get_BBAddr(void);
void Chip_Flush_Memory(void)
{
    static unsigned int u32_4Lines[64];
    volatile unsigned int *pu8;
    volatile unsigned int tmp ;  // Fixme: temporary mark this code.

    // Transfer the memory to noncache memory
    pu8 = ((volatile unsigned int *)(((unsigned int)u32_4Lines) | 0xa0000000));

    // Flush the data from pipe and buffer in MIU0
    pu8[0] = pu8[16] = pu8[32] = pu8[48] = 1;

    // Flush the data in the EC bridge buffer
    mb();

    // final read
    tmp = pu8[0] ;
    tmp = pu8[16] ;
    tmp = pu8[32] ;
    tmp = pu8[48] ;

    // Transfer the memory to noncache memory
    if(get_BBAddr()!=0)
    {
        pu8 =(volatile unsigned int*)(((unsigned int)get_BBAddr()) | 0xD0000000);
        // Flush the data from pipe and buffer in MIU1
        pu8[0] = pu8[16] = pu8[32] = pu8[48] = 1;

        // Flush the data in the EC bridge buffer
        mb();

        // final read
        tmp = pu8[0] ;
        tmp = pu8[16] ;
        tmp = pu8[32] ;
        tmp = pu8[48] ;

    }

}

void Chip_Read_Memory(void)
{
    volatile unsigned int *pu8;
    volatile unsigned int t ;
    // Transfer the memory to noncache memory
    pu8 = ((volatile unsigned int *)0xA0380000);
    t = pu8[0] ;
    t = pu8[64] ;
    // Fixme: temporary mark this code.
#if 1
    pu8 = ((volatile unsigned int *)0xD0380000);
    t = pu8[0] ;
    t = pu8[64] ;
#endif
}


unsigned int Chip_Query_MIPS_CLK(void)
{
    unsigned int u32Count = 0;
    unsigned int uSpeed_L = 0, uSpeed_H = 0;

    uSpeed_L = *(volatile u32*)(0xBF221864);
    uSpeed_H = *(volatile u32*)(0xBF221868);

    if(uSpeed_H==0x1c)
    {
        //printk("(CPU Speed is 12 MHz)\n");
        u32Count = 12000000;
    }
    else if(uSpeed_L==0x14)
    {
        //printk("(CPU Speed %d MHz)\n", (int)(uSpeed_H*24));
        u32Count = uSpeed_H*24*1000000;
    }
    else if(uSpeed_L==0x18)
    {
        //printk("(CPU Speed %d MHz setting)\n", (int)(uSpeed_H*48));
        u32Count = uSpeed_H*48*1000000;
    }

    return u32Count;
}

EXPORT_SYMBOL(Chip_Flush_Memory);
EXPORT_SYMBOL(Chip_Read_Memory);

#define ONE_SEC 1000

#define MAIL_ADDRESS                (0x33A0 << 1)//(3390 << 1)

#define MAIL_EVENT_SET_MSATER       0x99
#define MAIL_EVENT_SET_SECURE       0x66
#define MAIL_EVENT_GET_KEY          0x88
//#define MAIL_EVENT_PROC             0x88
#define MAIL_EVENT_CLEAR            0x77

#define MAIL_CONTENT_GET(_x_)       (((_x_) & 0xFF00) >> 8)
#define MAIL_CONTENT_SET(_x_, _v_)  (((_x_) & 0x00FF) | ((_v_) << 8))

/* ============================================================================== */
extern void Chip_Read_Memory(void);
extern void Chip_Flush_Memory(void);

//--------------------------------------------------------------------------------------------------
//  regAESDMA.h
//--------------------------------------------------------------------------------------------------

#define REG_AESDMACTRL_BASE         (0xBF200000 + 0x1940)  //(0x0C00/2 + 0x50) * 4

typedef unsigned char   MS_BOOL;
typedef unsigned char   MS_U8;
typedef unsigned short  MS_U16;
typedef unsigned int    MS_U32;
typedef MS_U32          DRVAESDMA_RESULT;
typedef MS_U32          APIAESDMA_RESULT;

#define DRVAESDMA_OK                   0x00000000
#define DRVAESDMA_FAIL                 0x00000001
#define DRVAESDMA_INVALID_PARAM        0x00000002
#define DRVAESDMA_FUNC_ERROR           0x00000003
#define DRVAESDMA_MIU_ADDR_ERROR       0x00000004

typedef struct {
    unsigned int u32SrcAddr;
    unsigned int u32SrcSize;
    unsigned int u32DstAddr;
    unsigned int u32DstSize;
    MS_BOOL    bMaster;
    MS_BOOL    bDecrypt;
} AESDMA_PARA;

typedef enum
{
    E_DRVAESDMA_CIPHER_ECB = 0,
    E_DRVAESDMA_CIPHER_CTR,
    E_DRVAESDMA_CIPHER_CBC,
} DrvAESDMA_CipherMode;


typedef struct _REG32
{
    volatile MS_U16     L;
    volatile MS_U16     empty_L;
    volatile MS_U16     H;
    volatile MS_U16     empty_H;
} REG32;

typedef struct _REG_CipherKey
{
    REG32               Key_L;
    REG32               Key_H;
} REG_CipherKey;

typedef struct _REG_InitVector
{
    REG32               IV_L;
    REG32               IV_H;
} REG_InitVector;

typedef struct _REG_AESDMACtrl
{
    REG32               Dma_Ctrl;                                   //0x50
    #define AESDMA_CTRL_FILEIN_START            0x00000100
    #define AESDMA_CTRL_FILEOUT_START           0x00000001
    #define AESDMA_CTRL_BANK_R                  0x00000020
    #define AESDMA_CTRL_BANK_W                  0x00000800

    #define AESDMA_INT_EN                       0x00400000
    #define AESDMA_ENG_PS_RELEASE               0x00010000
    #define AESDMA_ENG_PS_IN_EN                 0x00100000
    #define AESDMA_ENG_PS_OUT_EN                0x00200000
    #define AESDMA_INT_EN                       0x00400000
    #define AESDMA_ENG_AES_EN                   0x01000000
    #define AESDMA_ENG_DESCRYPT                 0x02000000          // 0:encrypt, 1:decrypt
    #define AESDMA_ENG_TDES_EN                  0x10000000          // If (aes_en) AES DMA Else if (tdes_en) TDES DMA Else DMA only
    #define AESDMA_ENG_KEY_SEL                  0x20000000          // 0: reg_cipher_key, 1: key from KL
    #define AESDMA_ENG_CTR_MODE                 0x10000000
    #define AESDMA_ENG_CBC_MODE                 0x20000000
    #define AESDMA_ECO_FIX_LAST_BYTE            0x80000000          // when (output address + length)/8=1 the last byte maybe no output

    #define AESDMA_ENG_PS_IN_EN                 0x00100000
    #define AESDMA_ENG_PS_OUT_EN                0x00200000
    #define AESDMA_ENG_AES_EN                   0x01000000
    #define AESDMA_ENG_DESCRYPT                 0x02000000          // 0:encrypt, 1:decrypt


    REG32               Dma_Filein_Addr;                            //0x52
    REG32               Dma_Filein_Num;                             //0x54
    REG32               Dma_Fileout_SAddr;                          //0x56
    REG32               Dma_Fileout_EAddr;                          //0x58
    REG32               Dma_PS_Pattern;                             //0x5a
    REG32               Dma_PS_Pattern_Mask;                        //0x5c
    REG32               Dma_Ctrl2;                                  //0x5e
    REG_CipherKey       Dma_CipherKey_L;                            //0x60
    REG_CipherKey       Dma_CipherKey_H;                            //0x64
    REG_InitVector      Dma_InitVector_L;                           //0x68
    REG_InitVector      Dma_InitVector_H;                           //0x6C
    REG32               Dma_Matched_Btyecnt;                        //0x70
    REG32               Dma_Matched_Pat;                            //0x72
    REG32               Dma_Err_Wadr;                               //0x74
    REG32               _xbf808fd8[4];                              //0x76~0x7c
    REG32               Dma_PVR_Status;                            //0x7e


/*
    #define AESDMA_PS_DONE                      0x00000001
    #define AESDMA_PS_STOP                      0x00000002
    #define AESDMA_DMA_DONE                     0x00010000
    #define AESDMA_DMA_PAUSE                    0x00020000
    #define AESDMA_STATES_GROUP                 (AESDMA_PS_DONE     | \
                                                 AESDMA_PS_STOP     | \
                                                 AESDMA_DMA_DONE    | \
                                                 AESDMA_DMA_PAUSE  )
*/
} REG_AESDMACtrl;


//--------------------------------------------------------------------------------------------------
//  halAESDMA.h
//--------------------------------------------------------------------------------------------------

#define HAS_FLAG(flag, bit)        ((flag) & (bit))
#define SET_FLAG(flag, bit)        ((flag)|= (bit))
#define RESET_FLAG(flag, bit)      ((flag)&= (~(bit)))
#define SET_FLAG1(flag, bit)       ((flag)|  (bit))
#define RESET_FLAG1(flag, bit)     ((flag)&  (~(bit)))

/// AESDMA Cipher Mode
typedef enum
{
    E_AESDMA_CIPHER_ECB = 0,
    E_AESDMA_CIPHER_CTR,
    E_AESDMA_CIPHER_CBC,
} AESDMA_CipherMode;



//--------------------------------------------------------------------------------------------------
//  halAESDMA.c
//--------------------------------------------------------------------------------------------------

static REG_AESDMACtrl * const _AESDMACtrl = (REG_AESDMACtrl *)REG_AESDMACTRL_BASE;

#define REG32_W(reg, value) do { \
                                (reg)->L = ((value) & 0x0000FFFF); \
                                (reg)->H = ((value) >> 16); \
                            } while(0)

#define REG32_W2(reg, value) do { \
                                (reg)->H = ((value) >> 16); \
                                printk("ctrl_H %x\n", _AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl));	\
                                (reg)->L = ((value) & 0x0000FFFF); \
                                printk("ctrl_L %x\n", _AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl));	\
                            } while(0)

#define REG32_W_L(reg, value)   do {    \
                                    (reg)->L = ((value) & 0x0000FFFF);  \
                                } while(0)

#define REG32_W_H(reg, value)   do {    \
                                    (reg)->H = ((value) >> 16); \
                                } while(0)


static MS_U32 _AESDMA_REG32_R(REG32 *reg)
{
    MS_U32     value;

    value = (reg)->H << 16;
    value |= (reg)->L;
    return value;
}

void AESDMA_Reset(void)
{
    MS_U32 Reg_AESDMA;
    int i = 0;

	//printk("AESDMA_Reset\n");


    Reg_AESDMA = (MS_U32)(&_AESDMACtrl[0].Dma_Ctrl);
    REG32_W((&_AESDMACtrl[0].Dma_Ctrl) , 0x00000080); // sw rst

    // reset all AESdma register
    for (i = 0 ; i < 19 ; i++)
    {
        REG32_W((REG32 *)(Reg_AESDMA+(i*8)), 0x00000000);
    }
}

void AESDMA_Set_CipherKey(MS_U32 *cipherkey)
{
    REG32_W((&_AESDMACtrl[0].Dma_CipherKey_L.Key_L) , cipherkey[0]);
    REG32_W((&_AESDMACtrl[0].Dma_CipherKey_L.Key_H) , cipherkey[1]);
    REG32_W((&_AESDMACtrl[0].Dma_CipherKey_H.Key_L) , cipherkey[2]);
    REG32_W((&_AESDMACtrl[0].Dma_CipherKey_H.Key_H) , cipherkey[3]);
}


static void AESDMA_Sel_Key(MS_BOOL keysel)
{
    MS_U32 u32keysel;
    u32keysel = (keysel<<29)&AESDMA_ENG_KEY_SEL;
    REG32_W((&_AESDMACtrl[0].Dma_Ctrl), _AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl)|u32keysel);
}

static void AESDMA_Set_FileinDesc(MS_U32 FileinAddr , MS_U32 FileinNum)
{
    REG32_W((&_AESDMACtrl[0].Dma_Filein_Addr), FileinAddr);
    REG32_W((&_AESDMACtrl[0].Dma_Filein_Num), FileinNum);
}

static void AESDMA_Set_FileoutDesc(MS_U32 FileoutSAddr , MS_U32 FileoutEAddr)
{
    REG32_W((&_AESDMACtrl[0].Dma_Fileout_SAddr), FileoutSAddr);
    REG32_W((&_AESDMACtrl[0].Dma_Fileout_EAddr), FileoutEAddr);
}

static void AESDMA_Start(MS_BOOL AESDMAStart)
{
	//printk("AESDMA_Start\n");
    if (AESDMAStart)
    {
        //REG32_W((&_AESDMACtrl[0].Dma_Ctrl), (_AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl) & ~(AESDMA_CTRL_FILEOUT_START | AESDMA_CTRL_FILEIN_START)));
        REG32_W_H((&_AESDMACtrl[0].Dma_Ctrl), (_AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl) | AESDMA_ECO_FIX_LAST_BYTE));
        REG32_W_L((&_AESDMACtrl[0].Dma_Ctrl), (_AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl) | AESDMA_CTRL_FILEOUT_START | AESDMA_CTRL_FILEIN_START));
    }
    else
    {
        REG32_W((&_AESDMACtrl[0].Dma_Ctrl) , _AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl)& ~(AESDMA_CTRL_FILEOUT_START | AESDMA_CTRL_FILEIN_START));
    }
}

static MS_U32 AESDMA_Get_AESDMA_Status(void)
{
	//printk("AESDMA_Get_AESDMA_Status\n");
    return _AESDMA_REG32_R(&_AESDMACtrl[0].Dma_PVR_Status);
}

static void AESDMA_Set_CIPHER_ENG(AESDMA_CipherMode eMode, MS_BOOL Descrypt)
{
    MS_U32 u32Cmd = 0;

    switch (eMode)
    {
    case E_AESDMA_CIPHER_ECB:
        u32Cmd |= AESDMA_ENG_AES_EN;
        break;

    case E_AESDMA_CIPHER_CTR:
        u32Cmd |= (AESDMA_ENG_AES_EN | AESDMA_ENG_CTR_MODE);
        break;

    case E_AESDMA_CIPHER_CBC:
        u32Cmd |= (AESDMA_ENG_AES_EN | AESDMA_ENG_CBC_MODE);
        break;

    default:
        return;
    }

    u32Cmd |= AESDMA_ENG_DESCRYPT & (Descrypt<<25);

    REG32_W((&_AESDMACtrl[0].Dma_Ctrl), _AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl) | u32Cmd);
}

static void AESDMA_Set_MIU_Path(MS_BOOL MIU_R, MS_BOOL MIU_W)
{
    REG32_W((&_AESDMACtrl[0].Dma_Ctrl), RESET_FLAG1(_AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl), AESDMA_CTRL_BANK_R));
    REG32_W((&_AESDMACtrl[0].Dma_Ctrl), RESET_FLAG1(_AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl), AESDMA_CTRL_BANK_W));

    REG32_W((&_AESDMACtrl[0].Dma_Ctrl), SET_FLAG1(_AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl), (AESDMA_CTRL_BANK_R & ((MS_U32)MIU_R << 5))));
    REG32_W((&_AESDMACtrl[0].Dma_Ctrl), SET_FLAG1(_AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl), (AESDMA_CTRL_BANK_W & ((MS_U32)MIU_W << 11))));

	//printk("AESDMA_Set_MIU_Path %x\n", _AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl));
}



//--------------------------------------------------------------------------------------------------
//  drvAESDMA.c
//--------------------------------------------------------------------------------------------------

#define AESDMAPA2KSEG1(addr) ((void *)(((MS_U32)addr) | 0xa0000000)) //physical -> unchched

MS_U32 _u32MIU0_Addr, _u32MIU1_Addr, _u32MIU_num;

static MS_BOOL _AESDMA_Chk_MIU(MS_U32 u32InAddr, MS_U32 u32InLen, MS_U32 u32OutSAddr, MS_U32 u32OutEAddr)
{
    u32InAddr = (MS_U32)AESDMAPA2KSEG1(u32InAddr);
    u32OutSAddr = (MS_U32)AESDMAPA2KSEG1(u32OutSAddr);
    u32OutEAddr = (MS_U32)AESDMAPA2KSEG1(u32OutEAddr);

    if (_u32MIU_num == 2)
    {
        if ((_u32MIU1_Addr > u32InAddr) & (_u32MIU1_Addr > u32InAddr+u32InLen) & (_u32MIU1_Addr > u32OutSAddr) & (_u32MIU1_Addr > u32OutEAddr))
        {
            AESDMA_Set_MIU_Path(false,false); // miu0->miu0
            return true;
        }

        if ((_u32MIU1_Addr < u32InAddr) & (_u32MIU1_Addr < u32InAddr+u32InLen) & (_u32MIU1_Addr < u32OutSAddr) & (_u32MIU1_Addr < u32OutEAddr))
        {
            AESDMA_Set_MIU_Path(true,true); // miu1->miu1
            return true;
        }

        if ((_u32MIU1_Addr > u32InAddr) & (_u32MIU1_Addr > u32InAddr+u32InLen) & (_u32MIU1_Addr <= u32OutSAddr) & (_u32MIU1_Addr <= u32OutEAddr))
        {
            AESDMA_Set_MIU_Path(false,true); // miu0->miu1
            return true;
        }

        if ((_u32MIU1_Addr <= u32InAddr) & (_u32MIU1_Addr <= u32InAddr+u32InLen) & (_u32MIU1_Addr > u32OutSAddr) & (_u32MIU1_Addr > u32OutEAddr))
        {
            AESDMA_Set_MIU_Path(true,false); // miu1->miu0
            return true;
        }
        return false;
    }
    else
    {
        return true;
    }
}

DRVAESDMA_RESULT MDrv_AESDMA_Init(MS_U32 u32miu0addr , MS_U32 u32miu1addr , MS_U32 u32miunum)
{
	//printk("MDrv_AESDMA_Init\n");
    _u32MIU_num = u32miunum;
    _u32MIU0_Addr = (MS_U32)AESDMAPA2KSEG1(u32miu0addr);

    if (_u32MIU_num == 2)
    {
        _u32MIU1_Addr = (MS_U32)AESDMAPA2KSEG1(u32miu1addr);
    }

    // ++ Release reset that asserted in the DC off
    // *(volatile MS_U32*)(0xbf800000+(0x2380<<2)) |= BIT15;
    // -- Release reset that asserted in the DC off

    AESDMA_Reset();

    REG32_W((&_AESDMACtrl[0].Dma_Ctrl) , _AESDMA_REG32_R(&_AESDMACtrl[0].Dma_Ctrl)|AESDMA_INT_EN);


    return DRVAESDMA_OK;
}

//#define MDrv_AESDMA_SetKey(pCipherKey)  AESDMA_Set_CipherKey(pCipherKey)

#define MDrv_AESDMA_Sel_Key(keysel)     AESDMA_Sel_Key(keysel)

#define MDrv_AESDMA_SetIV(pInitVector)  AESDMA_Set_InitVector((pInitVector))

#define MDrv_AESDMA_SelEng(eMode, bDescrypt) AESDMA_Set_CIPHER_ENG((eMode), ((E_DRVAESDMA_CIPHER_CTR == (eMode)) ? false : (bDescrypt)))

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

DRVAESDMA_RESULT MDrv_AESDMA_SetKey(MS_U32 *pCipherKey, MS_BOOL keysel)
{
    if (pCipherKey != NULL)
    {
        AESDMA_Set_CipherKey(pCipherKey);
        AESDMA_Sel_Key(FALSE); // key from cipherkey
    }
    else
    {
        AESDMA_Sel_Key(TRUE); // key from KL
    }

    return DRVAESDMA_OK;
}


DRVAESDMA_RESULT MDrv_AESDMA_SetFileInOut(MS_U32 u32FileinAddr, MS_U32 u32FileInNum, MS_U32 u32FileOutSAddr, MS_U32 u32FileOutEAddr)
{
	//printk("MDrv_AESDMA_SetFileInOut\n");
    // check miu addr
    if (!_AESDMA_Chk_MIU(u32FileinAddr, u32FileInNum, u32FileOutSAddr, u32FileOutEAddr))
    {
        return DRVAESDMA_MIU_ADDR_ERROR;
    }

    if ((u32FileOutSAddr == 0) | (u32FileOutEAddr == 0))
    {
        AESDMA_Set_FileinDesc(u32FileinAddr , u32FileInNum);
    }
    else
    {
        AESDMA_Set_FileinDesc(u32FileinAddr, u32FileInNum);
        AESDMA_Set_FileoutDesc(u32FileOutSAddr, u32FileOutEAddr);
    }

    return DRVAESDMA_OK;
}

#define MDrv_AESDMA_Start(bStart)       AESDMA_Start((bStart))

#define MDrv_AESDMA_Reset() AESDMA_Reset()

#define MDrv_AESDMA_GetStatus()         AESDMA_Get_AESDMA_Status()

#define MDrv_AESDMA_GetRandom(buf, num) (*(buf) = *(volatile unsigned int*)(0xBF203A38))



#define MsOS_DelayTask(ms) udelay((ms)*1000)



/* ============================================================================== */

MS_U32 u32MbxBank;      // for mbx
MS_U32 u32MbxBankSize;  // for mbx
MS_U32 u32PhyBufAddr;
MS_U32 u32PhyBufSize;

//#define USER_KEY
#define PM_SET_KEY

#ifdef USER_KEY
 static MS_U32 u32PhyUserKey;
 #define USER_KEY_LEN 0x20
#else
 #define USER_KEY_LEN 0x0
#endif

#ifdef PM_SET_KEY
 #define PM_KEY_LEN 0x20
#else
 #define PM_KEY_LEN 0x0
#endif

//static MS_U32 u32VirBufAddr;
//static MS_U32 u32VirBufSize;
#ifdef PM_SET_KEY
    static MS_U32 u32AesLoop=0;
#endif
#define MASTER_USER_KEY_OFFSET 16


#define CHIP_SETUP_ASSERT(_con)   do {\
                                         if (!(_con)) {\
                                             printk(KERN_CRIT "BUG at %s:%d assert(%s)\n",\
                                             __FILE__, __LINE__, #_con);\
                                             BUG();\
                                         }\
                                     } while (0)

/*
#define CHIP_SETUP_ASSERT()   do {\
                                     printk(KERN_CRIT "BUG at %s:%d assert(%s)\n",\
                                     __FILE__, __LINE__, #_con);\
                                     BUG();\
                                 } while (0)
*/
MS_U32 _MDrv_Kernel_GetSystemTime(void)
{
    return jiffies_to_msecs(jiffies);
}

APIAESDMA_RESULT MApi_AESDMA_Init(MS_U32 u32miu0addr, MS_U32 u32miu1addr, MS_U32 u32miunum)
{
#if 0
    MDrv_MMIO_GetBASE(&u32Bank, &u32BankSize, MS_MODULE_MBX);
#else
    u32MbxBank     = 0xBF200000;   // HAL_MMIO_NONPM_BASE
    u32MbxBankSize = 0x00025600;   // HAL_MMIO_NONPM_SIZE
#endif
    u32PhyBufAddr  = 0x330000 + PM_KEY_LEN;
    u32PhyBufSize  = 0xd0000 - 0x1000 - PM_KEY_LEN;

    return MDrv_AESDMA_Init(u32miu0addr, u32miu1addr, u32miunum);
}
EXPORT_SYMBOL(MApi_AESDMA_Init);

void MDrv_AESDMA_PM_SetKey(MS_BOOL bMaster)
{
    MS_U32 TimeOut = _MDrv_Kernel_GetSystemTime();
    volatile unsigned int *p = (volatile unsigned int *)(u32MbxBank+MAIL_ADDRESS);

    while (MAIL_CONTENT_GET(*p) == MAIL_EVENT_SET_MSATER ||
           MAIL_CONTENT_GET(*p) == MAIL_EVENT_SET_SECURE ||
           MAIL_CONTENT_GET(*p) == MAIL_EVENT_GET_KEY)
        ;

    if (bMaster)
    {
         *p = MAIL_CONTENT_SET(*p, MAIL_EVENT_SET_MSATER);
    }
    else
    {
        *p = MAIL_CONTENT_SET(*p, MAIL_EVENT_SET_SECURE);
    }

    while (MAIL_CONTENT_GET(*p) == MAIL_EVENT_SET_MSATER ||
           MAIL_CONTENT_GET(*p) == MAIL_EVENT_SET_SECURE ||
           MAIL_CONTENT_GET(*p) == MAIL_EVENT_GET_KEY)
    {
        if ((_MDrv_Kernel_GetSystemTime() - TimeOut) > 500)
        {
            CHIP_SETUP_ASSERT((_MDrv_Kernel_GetSystemTime() - TimeOut) > 500);
           // printk("Wait Mailbox return fail\n");
            break;
        }
    }

	//printk("MDrv_AESDMA_PM_SetKey 4\n");
}

void MDrv_AESDMA_PM_GetSecureKey(void)
{
    MS_U32 TimeOut = _MDrv_Kernel_GetSystemTime();
    volatile unsigned int *p = (volatile unsigned int *)(u32MbxBank+MAIL_ADDRESS);

    while (MAIL_CONTENT_GET(*p) == MAIL_EVENT_SET_MSATER ||
           MAIL_CONTENT_GET(*p) == MAIL_EVENT_SET_SECURE ||
           MAIL_CONTENT_GET(*p) == MAIL_EVENT_GET_KEY)
        ;
    *p = MAIL_CONTENT_SET(*p, MAIL_EVENT_GET_KEY);

    while (MAIL_CONTENT_GET(*p) == MAIL_EVENT_SET_MSATER ||
           MAIL_CONTENT_GET(*p) == MAIL_EVENT_SET_SECURE ||
           MAIL_CONTENT_GET(*p) == MAIL_EVENT_GET_KEY)
    {
        if ((_MDrv_Kernel_GetSystemTime() - TimeOut) > 500)
        {
            CHIP_SETUP_ASSERT((_MDrv_Kernel_GetSystemTime() - TimeOut) > 500);
           // printk("Wait Mailbox return fail\n");
            break;
        }
    }
}


void MDrv_AESDMA_WaitFinish(void)
{
    volatile unsigned int *p = (volatile unsigned int *)u32MbxBank+MAIL_ADDRESS;

    while (MAIL_CONTENT_GET(*p) == MAIL_EVENT_SET_MSATER ||
           MAIL_CONTENT_GET(*p) == MAIL_EVENT_SET_SECURE ||
           MAIL_CONTENT_GET(*p) == MAIL_EVENT_GET_KEY)
        ;
}

void Chip_Flush_Memory_for_aesdma(void)
{
    static unsigned int u32_4Lines[64];
    volatile unsigned int *pu8;
    volatile unsigned int tmp ;  // Fixme: temporary mark this code.

    // Transfer the memory to noncache memory
    pu8 = ((volatile unsigned int *)(((unsigned int)u32_4Lines) | 0xa0000000));
    // Flush the data from pipe and buffer in MIU0
    pu8[0] = pu8[16] = pu8[32] = pu8[48] = 1;
    // Flush the data in the EC bridge buffer
    mb();
    // final read
    tmp = pu8[0] ;
    tmp = pu8[16] ;
    tmp = pu8[32] ;
    tmp = pu8[48] ;
}

APIAESDMA_RESULT MApi_AES_DMA(AESDMA_PARA para)
{
    DRVAESDMA_RESULT    result = DRVAESDMA_OK;
    int i = 0;
    MS_U32              u32RemainSize = para.u32SrcSize;
    MS_U32              u32ProcSize;
    MS_U32              u32BufAddr = (u32PhyBufAddr | 0xA0000000);

    if (u32RemainSize & 0xF)
    {
        return DRVAESDMA_FAIL;
    }

    #ifdef PM_SET_KEY
    if(u32AesLoop==0)
    {
        MDrv_AESDMA_PM_GetSecureKey();
        u32AesLoop++;
    }
    #endif

    while (u32RemainSize)
    {
        // check size
        if (u32RemainSize > (u32PhyBufSize-PM_KEY_LEN-USER_KEY_LEN))
        {
            u32ProcSize = u32PhyBufSize-PM_KEY_LEN-USER_KEY_LEN;
            u32RemainSize -= u32ProcSize;
        }
        else
        {
            u32ProcSize = u32RemainSize;
            u32RemainSize = 0;
        }

        memcpy((MS_U8*)u32BufAddr, (MS_U8*)para.u32SrcAddr, u32ProcSize);
        Chip_Flush_Memory_for_aesdma();

        MDrv_AESDMA_Reset();

        result = MDrv_AESDMA_SetFileInOut(u32BufAddr, u32ProcSize, u32BufAddr+u32ProcSize, u32BufAddr+u32ProcSize*2-1);
        MDrv_AESDMA_SelEng(E_DRVAESDMA_CIPHER_ECB, para.bDecrypt);

        #ifdef PM_SET_KEY
            MDrv_AESDMA_PM_SetKey(para.bMaster);    // 8051 set key;
            MDrv_AESDMA_Sel_Key(FALSE);
        #endif
        MDrv_AESDMA_Start(true);

        while ((MDrv_AESDMA_GetStatus() & BIT(16)) == 0)
        {
            if (i++ >= 1000)
            {
                return DRVAESDMA_FAIL;
            }
            MsOS_DelayTask(1);
        }

        memcpy((MS_U8*)para.u32DstAddr, (MS_U8*)(u32BufAddr+u32ProcSize), u32ProcSize);
        Chip_Flush_Memory_for_aesdma();

        para.u32SrcAddr += u32ProcSize;
        para.u32DstAddr += u32ProcSize;
    }

    return DRVAESDMA_OK;
}


APIAESDMA_RESULT MApi_MK_AES_ECB_Enc(unsigned int u32SrcAddr,
                                     unsigned int u32InDataLength,
                                     unsigned int u32DstAddr,
                                     unsigned int *pu32OutDataLength)
{
    AESDMA_PARA param;
    param.u32SrcAddr = u32SrcAddr;
    param.u32SrcSize = u32InDataLength;
    param.u32DstAddr = u32DstAddr;
    param.u32DstSize = *pu32OutDataLength;
    param.bMaster = true;
    param.bDecrypt = false;

    return MApi_AES_DMA(param);
}
EXPORT_SYMBOL(MApi_MK_AES_ECB_Enc);

APIAESDMA_RESULT MApi_MK_AES_ECB_Dec(unsigned int u32SrcAddr,
                                     unsigned int u32InDataLength,
                                     unsigned int u32DstAddr,
                                     unsigned int *pu32OutDataLength)
{
    AESDMA_PARA param;
    param.u32SrcAddr = u32SrcAddr;
    param.u32SrcSize = u32InDataLength;
    param.u32DstAddr = u32DstAddr;
    param.u32DstSize = *pu32OutDataLength;
    param.bMaster = true;
    param.bDecrypt = true;

    return MApi_AES_DMA(param);
}
EXPORT_SYMBOL(MApi_MK_AES_ECB_Dec);


APIAESDMA_RESULT MApi_SK_AES_ECB_Enc(unsigned int u32SrcAddr,
                                     unsigned int u32InDataLength,
                                     unsigned int u32DstAddr,
                                     unsigned int *pu32OutDataLength)
{
    AESDMA_PARA param;
    param.u32SrcAddr = u32SrcAddr;
    param.u32SrcSize = u32InDataLength;
    param.u32DstAddr = u32DstAddr;
    param.u32DstSize = *pu32OutDataLength;
    param.bMaster = false;
    param.bDecrypt = false;

    return MApi_AES_DMA(param);
}
EXPORT_SYMBOL(MApi_SK_AES_ECB_Enc);

APIAESDMA_RESULT MApi_SK_AES_ECB_Dec(unsigned int u32SrcAddr,
                                     unsigned int u32InDataLength,
                                     unsigned int u32DstAddr,
                                     unsigned int *pu32OutDataLength)
{
    AESDMA_PARA param;
    param.u32SrcAddr = u32SrcAddr;
    param.u32SrcSize = u32InDataLength;
    param.u32DstAddr = u32DstAddr;
    param.u32DstSize = *pu32OutDataLength;
    param.bMaster = false;
    param.bDecrypt = true;

    return MApi_AES_DMA(param);
}
EXPORT_SYMBOL(MApi_SK_AES_ECB_Dec);

APIAESDMA_RESULT MApi_AES_CBC_Enc(unsigned char *key,
                                  unsigned char *iv,
                                  unsigned int u32SrcAddr,
                                  unsigned int u32InDataLength,
                                  unsigned int u32DstAddr,
                                  unsigned int *pu32OutDataLength)
{
    return DRVAESDMA_FAIL;
}
EXPORT_SYMBOL(MApi_AES_CBC_Enc);

void MApi_AES_ET_RAND(unsigned int *buf, int num)
{
    MDrv_AESDMA_GetRandom(buf, num);
    return;
}
