/*
 * Carsten Langgaard, carstenl@mips.com
 * Copyright (C) 2000 MIPS Technologies, Inc.  All rights reserved.
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 */
// #include <linux/config.h>
#include <linux/autoconf.h>
#include <linux/undefconf.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/ioport.h>
#include <linux/pci.h>
#include <linux/tty.h>

#include <linux/serial.h>
#include <linux/serial_core.h>
#include <linux/serial_8250.h>


#ifdef CONFIG_MTD
#include <linux/mtd/partitions.h>
#include <linux/mtd/physmap.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#endif

#include <asm/cpu.h>
#include <asm/bootinfo.h>
#include <asm/irq.h>
#include <asm/mips-boards/generic.h>
#include <asm/mips-boards/prom.h>
#include <asm/dma.h>
#include <asm/time.h>
#include <asm/traps.h>

#include "chip_int.h"
#include "hwreg_S5.h"
#ifdef CONFIG_VT
#include <linux/console.h>
#endif

#include "board/Board.h"

#if !defined(ENABLE) && !defined(DISABLE)
/// definition for ENABLE
#define ENABLE                      1
/// definition for DISABLE
#define DISABLE                     0
#endif

#if !defined(BIT0) && !defined(BIT1)
#define BIT0	                    0x00000001
#define BIT1	                    0x00000002
#define BIT2	                    0x00000004
#define BIT3	                    0x00000008
#define BIT4	                    0x00000010
#define BIT5	                    0x00000020
#define BIT6	                    0x00000040
#define BIT7	                    0x00000080
#define BIT8	                    0x00000100
#define BIT9	                    0x00000200
#define BIT10	                    0x00000400
#define BIT11	                    0x00000800
#define BIT12	                    0x00001000
#define BIT13	                    0x00002000
#define BIT14	                    0x00004000
#define BIT15  	                    0x00008000
#define BIT16                       0x00010000
#define BIT17                       0x00020000
#define BIT18                       0x00040000
#define BIT19                       0x00080000
#define BIT20                       0x00100000
#define BIT21                       0x00200000
#define BIT22                       0x00400000
#define BIT23                       0x00800000
#define BIT24                       0x01000000
#define BIT25                       0x02000000
#define BIT26                       0x04000000
#define BIT27                       0x08000000
#define BIT28                       0x10000000
#define BIT29                       0x20000000
#define BIT30                       0x40000000
#define BIT31                       0x80000000
#endif

const char display_string[] = "        LINUX ON Titania3     ";

/******************************************************************************/
/* Macro for bitwise                                                          */
/******************************************************************************/
#define _BITMASK(loc_msb, loc_lsb) ((1U << (loc_msb)) - (1U << (loc_lsb)) + (1U << (loc_msb)))
#define BITMASK(x) _BITMASK(1?x, 0?x)

// UART_SEL_TYPE
#define UART_SEL_HK51_UART0     0x00
#define UART_SEL_HK51_UART1     0x01
#define UART_SEL_VD51_UART0     0x02
#define UART_SEL_AEON           0x03
#define UART_SEL_TSP            0x04
#define UART_SEL_PIU            0x05
#define UART_SEL_PM51_UART0     0x06

//----------------------------------------------------------
//priority setting: 0:highest priority, 3:lowest priority
//----------------------------------------------------------
//MIU group priority
//----------------------------------------------------------
#define  MIU0_GROUP0_PRIORITY     2
#define  MIU0_GROUP1_PRIORITY     0
#define  MIU0_GROUP2_PRIORITY     1
#define  MIU0_GROUP3_PRIORITY     3

#define  MIU1_GROUP0_PRIORITY     3
#define  MIU1_GROUP1_PRIORITY     2
#define  MIU1_GROUP2_PRIORITY     1
#define  MIU1_GROUP3_PRIORITY     0


#define MIU0_GROUP_PRIORITY       ( ( MIU0_GROUP3_PRIORITY << 6 )| (MIU0_GROUP2_PRIORITY<<4)| (MIU0_GROUP1_PRIORITY<<2) | (MIU0_GROUP0_PRIORITY) )
#define MIU1_GROUP_PRIORITY       ( ( MIU1_GROUP3_PRIORITY << 6 )| (MIU1_GROUP2_PRIORITY<<4)| (MIU1_GROUP1_PRIORITY<<2) | (MIU1_GROUP0_PRIORITY) )

////////////////////////////////////////////////////////////////////////////////
// MIU assignment   0: MIU0, 1: MIU1  2:IP_
////////////////////////////////////////////////////////////////////////////////

//----------------------------------------
//
//----------------------------------------
//group_0
#define  RESERVED_for_MIU_SELF     0
#define  VIVALDI9_DSP_DECODER      0
#define  VIVALDI9_DSP_SE           0
#define  MIPS_W                    0
#define  MIPS_R                    0
#define  MAU0_W                    0
#define  MAU1_R                    0
#define  VD_MHEG5_DCACHE_RW        1
#define  VD_MHEG5_QDMA_RW          1
#define  GE_RW                     2
#define  HVD_RW                    1
#define  HVD_BBU_RW                1
#define  UHC0_RW                   0
#define  UHC1_RW                   0
#define  PIU_DMA_RW                2
#define  ADCDVIPLL_RW              0    //co_client with MIU

//group_1
#define  VIVALDI9_MAD              0
#define  VIVALDI9_DMA              0
#define  VIVALDI9_AU_DMA           0
#define  MVOP_RW                   1   //mvop_128
#define  GOP_W                     0
#define  GOP0_R                    0  //may be changed later in miu_assignment
#define  GOP1_R                    0
#define  GOP2_R                    1
#define  GOP3_R                    0
#define  SC_OP_R                   0
#define  SC_IP_MIAN_W              0
#define  SC_IP_MIAN_R              0
#define  VD_COMB_W                 0
#define  VD_COMB_R                 0
#define  VE_W                      0
#define  VE_R                      0

//backward competiable
#define   MIUSEL_FLAG_DNRB_R        SC_IP_MIAN_R
#define   MIUSEL_FLAG_DNRB_RW       (SC_IP_MIAN_R | SC_IP_MIAN_W)


//group_2
#define  OD_W                      0
#define  OD_R                      0
#define  OD_LSB_W                  0
#define  OD_LSB_R                  0
#define  SC_IP_SUB_W               0
#define  SC_IP_SUB_R               0
#define  DISP_IPATH_NR_RW          0
#define  DISP_IPATH_MR_RW          0
#define  DISP_IPATH_DI_W           0
#define  VD_MHEG5_ICACHE           1
#define  TSP_W                     0
#define  TSP_R                     0
#define  VD_TTX_RW                 0
#define  VD_TTXSL_W                0
#define  TSP_ORZ_W                 0
#define  TSP_ORZ_R                 0

//group_3
#define  MVOP_64_R                 1
#define  M4VE_0_RW                 0
#define  M4VE_ME_R                 0
#define  M4VE_2                    0
#define  MVD_RW                    1
#define  MVD_BBU_RW                1
#define  RVD_RW                    1
#define  RVD_BBU_RW                1
#define  JPD_RW                    0
#define  DSCRMB_RW                 0
#define  STRLD_RW                  0
#define  FUART_RW                  0
#define  EMAC_RW                   0
#define  NAND_RW                   0
#define  MPIF_RW                   2   // MIU assignment   0: MIU0, 1: MIU1  2:IP_ASSIGN
#define  RESERVED_G3D_RW           0

#define  ClientGruop0_in_MIU1     ((( VIVALDI9_DSP_DECODER == 1)? 0x0002 : 0  )| \
                                   (( VIVALDI9_DSP_SE      == 1)? 0x0004 : 0  )| \
                                   (( MIPS_W               == 1)? 0x0008 : 0  )| \
                                   (( MIPS_R               == 1)? 0x0010 : 0  )| \
                                   (( MAU0_W               == 1)? 0x0020 : 0  )| \
                                   (( MAU1_R               == 1)? 0x0040 : 0  )| \
                                   (( VD_MHEG5_DCACHE_RW   == 1)? 0x0080 : 0  )| \
                                   (( VD_MHEG5_QDMA_RW     == 1)? 0x0100 : 0  )| \
                                   (( GE_RW                == 1)? 0x0200 : 0  )| \
                                   (( HVD_RW               == 1)? 0x0400 : 0  )| \
                                   (( HVD_BBU_RW           == 1)? 0x0800 : 0  )| \
                                   (( UHC0_RW              == 1)? 0x1000 : 0  )| \
                                   (( UHC1_RW              == 1)? 0x2000 : 0  )| \
                                   (( PIU_DMA_RW           == 1)? 0x4000 : 0  )| \
                                   (( ADCDVIPLL_RW         == 1)? 0x8000 : 0  )  )

#define  ClientGruop1_in_MIU1     (((VIVALDI9_MAD         == 1)? 0x0001 : 0  )| \
                                   ((VIVALDI9_DMA         == 1)? 0x0002 : 0  )| \
                                   ((VIVALDI9_AU_DMA      == 1)? 0x0004 : 0  )| \
                                   ((MVOP_RW              == 1)? 0x0008 : 0  )| \
                                   ((GOP_W                == 1)? 0x0010 : 0  )| \
                                   ((GOP0_R               == 1)? 0x0020 : 0  )| \
                                   ((GOP1_R               == 1)? 0x0040 : 0  )| \
                                   ((GOP2_R               == 1)? 0x0080 : 0  )| \
                                   ((GOP3_R               == 1)? 0x0100 : 0  )| \
                                   ((SC_OP_R              == 1)? 0x0200 : 0  )| \
                                   ((SC_IP_MIAN_W         == 1)? 0x0400 : 0  )| \
                                   ((SC_IP_MIAN_R         == 1)? 0x0800 : 0  )| \
                                   ((VD_COMB_W            == 1)? 0x1000 : 0  )| \
                                   ((VD_COMB_R            == 1)? 0x2000 : 0  )| \
                                   ((VE_W                 == 1)? 0x4000 : 0  )| \
                                   ((VE_R                 == 1)? 0x8000 : 0  )  )


#define  ClientGruop2_in_MIU1     (((OD_W                 == 1)? 0x0001 : 0 )| \
                                   ((OD_R                 == 1)? 0x0002 : 0 )| \
                                   ((OD_LSB_W             == 1)? 0x0004 : 0 )| \
                                   ((OD_LSB_R             == 1)? 0x0008 : 0 )| \
                                   ((SC_IP_SUB_W          == 1)? 0x0010 : 0 )| \
                                   ((SC_IP_SUB_R          == 1)? 0x0020 : 0 )| \
                                   ((DISP_IPATH_NR_RW     == 1)? 0x0040 : 0 )| \
                                   ((DISP_IPATH_MR_RW     == 1)? 0x0080 : 0 )| \
                                   ((DISP_IPATH_DI_W      == 1)? 0x0100 : 0 )| \
                                   ((VD_MHEG5_ICACHE      == 1)? 0x0200 : 0 )| \
                                   ((TSP_W                == 1)? 0x0400 : 0 )| \
                                   ((TSP_R                == 1)? 0x0800 : 0 )| \
                                   ((VD_TTX_RW            == 1)? 0x1000 : 0 )| \
                                   ((VD_TTXSL_W           == 1)? 0x2000 : 0 )| \
                                   ((TSP_ORZ_W            == 1)? 0x4000 : 0 )| \
                                   ((TSP_ORZ_R            == 1)? 0x8000 : 0 ))

#define  ClientGruop3_in_MIU1     (((MVOP_64_R            == 1)? 0x0001 : 0  )| \
                                   ((M4VE_0_RW            == 1)? 0x0002 : 0  )| \
                                   ((M4VE_ME_R            == 1)? 0x0004 : 0  )| \
                                   ((M4VE_2               == 1)? 0x0008 : 0  )| \
                                   ((MVD_RW               == 1)? 0x0010 : 0  )| \
                                   ((MVD_BBU_RW           == 1)? 0x0020 : 0  )| \
                                   ((RVD_RW               == 1)? 0x0040 : 0  )| \
                                   ((RVD_BBU_RW           == 1)? 0x0080 : 0  )| \
                                   ((JPD_RW               == 1)? 0x0100 : 0  )| \
                                   ((DSCRMB_RW            == 1)? 0x0200 : 0  )| \
                                   ((STRLD_RW             == 1)? 0x0400 : 0  )| \
                                   ((FUART_RW             == 1)? 0x0800 : 0  )| \
                                   ((EMAC_RW              == 1)? 0x1000 : 0  )| \
                                   ((NAND_RW              == 1)? 0x2000 : 0  )| \
                                   ((MPIF_RW              == 1)? 0x4000 : 0  )| \
                                   ((RESERVED_G3D_RW      == 1)? 0x8000 : 0  ))
//-----------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
#define  ClientGruop0_in_IP      (((VIVALDI9_DSP_DECODER == 2)? 0x0002:0 )|\
                                  ((VIVALDI9_DSP_SE      == 2)? 0x0004:0 )| \
                                  ((GE_RW                == 2)? 0x0200:0 )| \
                                  ((UHC0_RW              == 2)? 0x1000:0 )| \
                                  ((UHC1_RW              == 2)? 0x2000:0 )| \
                                  ((PIU_DMA_RW           == 2)? 0x4000:0 ))


#define  ClientGruop1_in_IP     (((VIVALDI9_MAD         == 2)? 0x0001 : 0 )| \
                                 ((VIVALDI9_DMA         == 2)? 0x0002 : 0 )| \
                                 ((VIVALDI9_AU_DMA      == 2)? 0x0004 : 0 )| \
                                 ((SC_OP_R              == 2)? 0x0100 : 0 )| \
                                 ((SC_IP_MIAN_W         == 2)? 0x0200 : 0 )| \
                                 ((SC_IP_MIAN_R         == 2)? 0x0800 : 0 )  )


#define  ClientGruop2_in_IP      ((  SC_IP_SUB_W         == 2)? 0x0010 : 0 | \
                                  (  SC_IP_SUB_R         == 2)? 0x0020 : 0)

#define  ClientGruop3_in_IP      ((  DSCRMB_RW           == 2)? 0x0100 : 0 | \
                                  (  STRLD_RW            == 2)? 0x0200 : 0 | \
                                  (  EMAC_RW             == 2)? 0x1000 : 0 | \
                                  (  NAND_RW             == 2)? 0x2000 : 0 | \
                                  (  MPIF_RW             == 2)? 0x4000 : 0 )
//-----------------------------------------------------------------------------

extern void mips_reboot_setup(void);
extern void mips_time_init(void);
extern void mips_timer_setup(struct irqaction *irq);
extern unsigned long mips_rtc_get_time(void);

static void __init serial_init(void);
static void __init miu_priority_config(void);
static void __init miu_assignment(void);


const char *get_system_type(void)
{
	return "MStar Titania3";
}

extern int MDrv_SYS_GetMMAP(int type, unsigned int *addr, unsigned int *len) ;

static int __init Titania3_setup(void)
{
       // int i;

	printk("Begin Titania3_setup\n");

    (*(volatile unsigned char *)(0xBF2025C0))=(unsigned char)(0x0);//for SWUpdateDelayCode for T3 only.

#if defined(CONFIG_MSTAR_TITANIA3)

#if defined(CONFIG_MSTAR_TITANIA3_MMAP_32MB_32MB) || \
    defined(CONFIG_MSTAR_TITANIA3_MMAP_64MB_32MB) || \
    defined(CONFIG_MSTAR_TITANIA3_MMAP_64MB_64MB) || \
    defined(CONFIG_MSTAR_TITANIA3_MMAP_128MB_64MB) || \
    defined(CONFIG_MSTAR_MMAP_128MB_128MB_DEFAULT) || \
    defined(CONFIG_MSTAR_TITANIA3_MMAP_128MB_128MB)

    // for MIU assignment
    // @FIXME: Richard correct miu assignment later
    //miu_assignment();
#endif

#endif

    //miu_priority_config();
	serial_init();

	mips_reboot_setup();

	return 0;
}

void __init plat_mem_setup(void)
{
    Titania3_setup();
}

extern void Uart16550IntrruptEnable(void);
extern void Uart16550Put(unsigned char);
static void __init serial_init(void)
{
#ifdef CONFIG_SERIAL_8250
	struct uart_port s,u;

	memset(&s, 0, sizeof(s));

    //Uart16550IntrruptEnable();
	//while(1) Uart16550Put('*');
	s.type = PORT_16550;
	s.iobase = 0xbf201300;
	// s.irq = E_IRQ_UART;
	s.irq = E_IRQ_UART0;

#ifdef CONFIG_MSTAR_TITANIA3

	// for FPGA
	// s.uartclk   = 12000000;
	//s.uartclk = 14318180;
	s.uartclk = 123000000;
	//s.uartclk = 144000000;
	//s.uartclk = 160000000;
	//s.uartclk = 172800000;
	//s.uartclk = 108000000;

#else

#ifdef CONFIG_MSTAR_TITANIA3_BD_FPGA
	//s.uartclk = FPGA_XTAL_CLOCK;    //12000000;///14318180;   //43200000;
	s.uartclk = 12000000 ;
#else
	s.uartclk = 123000000;
#endif

#endif
	s.iotype = 0;
	s.regshift = 0;
	s.fifosize = 16 ; // use the 8 byte depth FIFO well

	if (early_serial_setup(&s) != 0) {
		printk(KERN_ERR "Serial setup failed!\n");
	}

	// ------->hank.lai 2010.2.3
	memset(&u, 0, sizeof(u));

	u.type = PORT_16550;
	u.iobase = 0xBF220C00;
	u.irq = E_IRQEXPL_UART1;
	u.uartclk = 123000000;
	u.iotype = 0;
	u.regshift = 0;
	u.fifosize = 16 ; // use the 8 byte depth FIFO well

	u.line = 1;
	
	if (early_serial_setup(&u) != 0) {
		printk(KERN_ERR "Serial piu_uart1 setup failed!\n");
	}
	// <-------
#endif
}

#define _MapBase_nonPM 0xbf200000
#define _MapBase_PM    0xbf000000

static unsigned int u32MapBase = _MapBase_nonPM;

static void __init _RV1(unsigned int addr, unsigned short value)
{
    if(addr & 1)
        (*(volatile unsigned char *)(u32MapBase+((addr-1)<<1)+1))=(unsigned char)(value & 0xFF);
    else
        (*(volatile unsigned char *)(u32MapBase+(addr<<1)))=(unsigned char)(value & 0xFF);
}

static void __init _RV2(unsigned int addr, unsigned short value)
{
    (*(volatile unsigned short *)(u32MapBase+(addr<<1)))=(unsigned short)(value & 0xFFFF);
}


// The following macros are defined just for easy porting from 8051 code
#define MDrv_WriteByte(addr, value)     _RV1(addr, value)
#define MDrv_Write2Byte(addr, value)    _RV2(addr, value)

#define MST_MACRO_START     do {
#define MST_MACRO_END       } while (0)

#define RIU     ((unsigned short volatile *) u32MapBase)
#define RIU8    ((unsigned char  volatile *) u32MapBase)

#define MDrv_WriteWordRegBit( u32Reg, bEnable, u16Mask )                                     \
    MST_MACRO_START                                                                     \
    RIU[u32Reg] = (bEnable) ? (RIU[u32Reg] |  (u16Mask)) :                           \
                                (RIU[u32Reg] & ~(u16Mask));                            \
    MST_MACRO_END


#define    RIUBASE_MIU                 0x101200    //0x101200 ~ 0x1012ff
#define    RIUBASE_MIU2                0x100600    //0x100600 ~ 0x1006ff
static void __init miu_priority_config(void)
{
    u32MapBase = _MapBase_PM;

   //set group priority in MIU_0
   MDrv_WriteWordRegBit( ( RIUBASE_MIU+0xDC ),DISABLE, BIT8 );
   //printf("MIU0_GROUP_PRIORITY=%x",MIU0_GROUP_PRIORITY );
   MDrv_WriteByte( ( RIUBASE_MIU+0xDC ), MIU0_GROUP_PRIORITY );
   MDrv_WriteWordRegBit( ( RIUBASE_MIU+0xDC ),ENABLE, BIT8 );

   //set group priority in MIU_1
   MDrv_WriteWordRegBit( ( RIUBASE_MIU2+0xDC ),DISABLE, BIT8 );
   MDrv_WriteByte( ( RIUBASE_MIU2+0xDC ), MIU1_GROUP_PRIORITY );
   MDrv_WriteWordRegBit( ( RIUBASE_MIU2+0xDC ),ENABLE, BIT8 );
}


static int GOP0_MIU = 0;
static int GOP1_MIU = 0;
static int GOP2_MIU = 0;
static int GOP3_MIU = 0;

static int __init GOP0_MIU_setup(char *str)
{
    if( str != NULL )
    {
        sscanf(str,"%d",&GOP0_MIU);
    }

    return 0;
}

static int __init GOP1_MIU_setup(char *str)
{
    if( str != NULL )
    {
        sscanf(str,"%d",&GOP1_MIU);
    }

    return 0;
}

static int __init GOP2_MIU_setup(char *str)
{
    if( str != NULL )
    {
        sscanf(str,"%d",&GOP2_MIU);
    }

    return 0;
}

static int __init GOP3_MIU_setup(char *str)
{
    if( str != NULL )
    {
        sscanf(str,"%d",&GOP3_MIU);
    }

    return 0;
}

early_param("MS_GOP0_MIU", GOP0_MIU_setup);
early_param("MS_GOP1_MIU", GOP1_MIU_setup);
early_param("MS_GOP2_MIU", GOP2_MIU_setup);
early_param("MS_GOP3_MIU", GOP3_MIU_setup);

static void __init miu_assignment(void)
{
    u32MapBase = _MapBase_PM;

   MDrv_Write2Byte( ( RIUBASE_MIU+0xf0 ), ClientGruop0_in_MIU1 );   //test
   MDrv_Write2Byte( ( RIUBASE_MIU+0xf2 ), ClientGruop1_in_MIU1  );
   MDrv_Write2Byte( ( RIUBASE_MIU+0xf4 ), ClientGruop2_in_MIU1  );
   MDrv_Write2Byte( ( RIUBASE_MIU+0xf6 ), ClientGruop3_in_MIU1  );

   MDrv_Write2Byte( ( RIUBASE_MIU2+0xf0 ), ClientGruop0_in_IP  );
   MDrv_Write2Byte( ( RIUBASE_MIU2+0xf2 ), ClientGruop1_in_IP  );
   MDrv_Write2Byte( ( RIUBASE_MIU2+0xf4 ), ClientGruop2_in_IP  );
   MDrv_Write2Byte( ( RIUBASE_MIU2+0xf6 ), ClientGruop3_in_IP  );

    //EMAC access miu1
    MDrv_WriteWordRegBit(( RIUBASE_MIU+0x1C ), ENABLE, BIT6);

    //GOP0 access miu1
    if(GOP0_MIU!=0)
    {
        MDrv_WriteWordRegBit(( RIUBASE_MIU+0xf2 ), ENABLE, BIT5);
    }

    //GOP1 access miu1
    if(GOP1_MIU!=0)
    {
        MDrv_WriteWordRegBit(( RIUBASE_MIU+0xf2 ), ENABLE, BIT6);
    }

    //GOP2 access miu1
    if(GOP2_MIU!=0)
    {
        MDrv_WriteWordRegBit(( RIUBASE_MIU+0xf2 ), ENABLE, BIT7);
    }

    //GOP3 access miu1
    if(GOP3_MIU!=0)
    {
        MDrv_WriteWordRegBit(( RIUBASE_MIU+0xf2 ), ENABLE, BIT8);
    }
}
//this table is only for T3, not compatible with T2/U3
#define UART_REG(addr)          (*(volatile unsigned int*)((addr)))
#define TITANIA3_UART_BASE        0xbf201300
#define TITANIA3_UART_RX          (0x00*4 + TITANIA3_UART_BASE)
#define TITANIA3_UART_TX          (0x00*4 + TITANIA3_UART_BASE)
#define TITANIA3_UART_DLL         (0x00*4 + TITANIA3_UART_BASE)
#define TITANIA3_UART_DLM         (0x02*4 + TITANIA3_UART_BASE)
#define TITANIA3_UART_IER         (0x02*4 + TITANIA3_UART_BASE)
#define TITANIA3_UART_IIR         (0x04*4 + TITANIA3_UART_BASE)
#define TITANIA3_UART_FCR         (0x04*4 + TITANIA3_UART_BASE)

#define TITANIA3_UART_LCR         (0x06*4 + TITANIA3_UART_BASE)
#define TITANIA3_UART_MCR         (0x08*4 + TITANIA3_UART_BASE)
#define TITANIA3_UART_LSR         (0x0A*4 + TITANIA3_UART_BASE)
#define TITANIA3_UART_MSR         (0x0C*4 + TITANIA3_UART_BASE)

//Baud rate
#define TITANIA3_UART_BAUD_RATE  115200
// #define TITANIA3_UART_BAUD_RATE  38400

//Clock source
// 12000000, 14318180 , 123000000 , 144000000 , 160000000 , 172800000 , 108000000
#define TITANIA3_UART_CLK_FREQ     123000000

//Divisor
#define TITANIA3_UART_DIVISOR    (TITANIA3_UART_CLK_FREQ/(16 * TITANIA3_UART_BAUD_RATE))


extern unsigned long get_BBAddr(void);
void Chip_Flush_Memory(void)
{
    static unsigned int u32_4Lines[64];
    volatile unsigned int *pu8;
    volatile unsigned int tmp ;  // Fixme: temporary mark this code.

    // Transfer the memory to noncache memory
    pu8 = ((volatile unsigned int *)(((unsigned int)u32_4Lines) | 0xa0000000));

    // Flush the data from pipe and buffer in MIU0
    pu8[0] = pu8[16] = pu8[32] = pu8[48] = 1;

    // Flush the data in the EC bridge buffer
    mb();

    // final read
    tmp = pu8[0] ;
    tmp = pu8[16] ;
    tmp = pu8[32] ;
    tmp = pu8[48] ;

    // Transfer the memory to noncache memory
    if(get_BBAddr()!=0)
    {
        pu8 =(volatile unsigned int*)(((unsigned int)get_BBAddr()) | 0xC0000000);
        // Flush the data from pipe and buffer in MIU1
        pu8[0] = pu8[16] = pu8[32] = pu8[48] = 1;

        // Flush the data in the EC bridge buffer
        mb();

        // final read
        tmp = pu8[0] ;
        tmp = pu8[16] ;
        tmp = pu8[32] ;
        tmp = pu8[48] ;

    }

}

void Chip_Read_Memory(void)
{
    volatile unsigned int *pu8;
    volatile unsigned int t ;

    // Transfer the memory to noncache memory
    pu8 = ((volatile unsigned int *)0xA0380000);
    t = pu8[0] ;
    t = pu8[64] ;

    // Fixme: temporary mark this code.
#if 1
    pu8 = ((volatile unsigned int *)0xC0380000);
    t = pu8[0] ;
    t = pu8[64] ;
#endif
}


unsigned int Chip_Query_MIPS_CLK(void)
{
    unsigned int u32Count = 0;
    unsigned int uSpeed_L = 0, uSpeed_H = 0;

    uSpeed_L = *(volatile u32*)(0xBF221864);
    uSpeed_H = *(volatile u32*)(0xBF221868);

    if(uSpeed_H==0x1c)
    {
        //printk("(CPU Speed is 12 MHz)\n");
        u32Count = 12000000;
    }
    else if(uSpeed_L==0x14)
    {
        //printk("(CPU Speed %d MHz)\n", (int)(uSpeed_H*24));
        u32Count = uSpeed_H*24*1000000;
    }
    else if(uSpeed_L==0x18)
    {
        //printk("(CPU Speed %d MHz setting)\n", (int)(uSpeed_H*48));
        u32Count = uSpeed_H*48*1000000;
    }

    return u32Count;
}

