/*
 * kdebugd.h
 *
 * Copyright (C) 2009 Samsung Electronics
 * Created by lee jung-seung (js07.lee@samsung.com)
 *
 * NOTE:
 *
 */

#ifndef __KDEBUGD_H__
#define __KDEBUGD_H__

#include<linux/sched.h>

/*      
 * Events (results of Get Kdebugd Event)
 */     

typedef unsigned int debugd_event_t;
#define DEBUGD_MAX_EVENTS         64 
#define DEBUGD_MAX_CHARS          16
struct debugd_queue {
	unsigned int            event_head;
	unsigned int            event_tail;
	debugd_event_t		events[DEBUGD_MAX_EVENTS];
};

#ifdef CONFIG_KDEBUGD_FUTEX
struct futex_waiter
{
	int hash_key;
	int wait_msec;
	struct task_struct *task;
};
#endif

extern char sched_serial[10];
extern void dump_pid_maps(struct task_struct* task);

extern unsigned int kdebugd_running;
extern struct debugd_queue kdebugd_queue;
extern int kdebugd_start(void);
extern void queue_add_event(struct debugd_queue *q, debugd_event_t event);

extern unsigned long kdbg_get_uptime(void);

extern unsigned int debugd_get_event(void);
extern int kdbg_register(char* name,int (*func)(void), void (*turnoff)(void));

extern void task_state_help(void);

extern struct task_struct *find_user_task_with_pid(void);

int kdebugd_status(void);

#ifdef CONFIG_TASK_STATE_BACKTRACE
void kdbg_unwind_mem_stack_kernel(const char *str, unsigned long bottom, unsigned long top);
#endif

void kdbg_dump_mem(const char *str, unsigned long bottom, unsigned long top);

/* Look up the first VMA which satisfies  addr < vm_end,  NULL if none. */
struct vm_area_struct*
find_vma2(struct mm_struct * mm, unsigned long addr);

void kdgb_set_print_status(int status);

struct kdbg_mem_watch {                                                                                              
    unsigned int pid;                                                                                           
    unsigned int tgid;                                                                                          
    unsigned int u_addr;                                                                                        
    unsigned int buff;                                                                                          
    unsigned int watching;                                                                                      
};                                                                                                              

#ifdef CONFIG_KDGB_SUPPRESS_PRINTK
extern int kdbg_outside_print;
#define PRINT_KD_STR     "<D>"
#else
#define PRINT_KD_STR     
#endif

#define  PRINT_KD(fmt , args ...)  	 printk(PRINT_KD_STR fmt,##args) 

#ifdef CONFIG_KDEBUGD_COUNTER_MONITOR
typedef int (*counter_monitor_func_t)(void);

extern int register_counter_monitor_func(counter_monitor_func_t func);
#endif

#endif	// __KDEBUGD_H__

