
#include <linux/proc_fs.h>
#include <linux/syscalls.h>
#include "include/Secureboot.h"
#include "include/SBKN_CC_API.h"
#include "include/publicKey.h"

int getAuthUld(macAuthULd_t *mac_authuld)
{
	int fd;
	char strrootfs[20];
	
	strncpy(strrootfs, ROOTFS_PARTITION,19);

	fd = sys_open(strrootfs, O_RDONLY, 0);
	if(fd >= 0) 
	{
		sys_lseek(fd, -4096, SEEK_END);
		sys_read(fd, (void *)mac_authuld, sizeof(MacInfo_t));
		sys_close(fd);
        CIP_CRIT_PRINT("%x is authuld length : file open success(rootfs)  \n", mac_authuld->macAuthULD.msgLen);	                    
        if(mac_authuld->macAuthULD.msgLen > 0xFFFF) // authuld size error checking.
        {
            CIP_CRIT_PRINT("authuld length is over 0xFFFF: check if authuld sign is added in rootfs \n", mac_authuld->macAuthULD.msgLen);	                                
    		return 0;
        }
        else
        {        
		    return 1;
        }
	} 
	CIP_CRIT_PRINT("%s file open error(rootfs)\n", strrootfs);
	
	return 0;
}

int read_from_file_or_ff (int fd, unsigned char *buf, int size)
{
	int read_cnt;
	int i;

	read_cnt = sys_read(fd, buf, size);

	if( read_cnt <= 0 ) 
	{
		read_cnt = 0;
	}
	
	for(i=read_cnt; i<size; i++) 
	{
		buf[i] = 0xff;
	}
	
	return size;
}
int verify_mstar_rsa_signature(unsigned char *pText, unsigned long pTextLen, unsigned char *signature, unsigned long signLen) 
{
	cc_u8 GE[256] = {0x01, 0x00, 0x01};
	unsigned long NLen = 128, ELen = 3;
	int result;

	CryptoCoreContainer *crt;	

	if( pTextLen > 128 )
	{
		pTextLen = 128;
	}

	crt = create_CryptoCoreContainer(ID_RSA1024);
	crt->RSA_setKeypair(crt, ID_NO_PADDING, GN, NLen, GE, ELen, NULL, 0);
	crt->DS_verify(crt, pText, pTextLen, signature, signLen, &result);	
 	destroy_CryptoCoreContainer(crt);

	if (result == CRYPTO_VALID_SIGN) 
    {
		return CRYPTO_SUCCESS;
	}
	else 
    {
		return CRYPTO_ERROR;
	}
}
